/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
*/
/*------------------------------------------------------------------------------
* Low Level C Generic Routines
*------------------------------------------------------------------------------*/
#define _WIN32_WINNT 0x0400
#define WINVER 0x0400
#define _WIN32_IE 0x0501

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"




static LOGFONT mylf; 


HB_FUNC ( REGISTERFONT )
{
    mylf.lfHeight            =           hb_parni(1);   // height of font
    mylf.lfWidth             =           hb_parni(2);   // average character width
    mylf.lfEscapement        =           hb_parni(3);   // angle of escapement
    mylf.lfOrientation       =           hb_parni(4);   // base-line orientation angle
    mylf.lfWeight            =           hb_parni(5);   // font weight
    mylf.lfItalic            =           hb_parl(6);    // italic attribute option
    mylf.lfUnderline         =           hb_parl(7);    // underline attribute option
    mylf.lfStrikeOut         =           hb_parl(8);    // strikeout attribute option
    mylf.lfCharSet           =           hb_parnl(9);   // character set identifier
    mylf.lfOutPrecision      =           hb_parnl(10);  // output precision
    mylf.lfClipPrecision     =           hb_parnl(11);  // clipping precision
    mylf.lfQuality           =           hb_parnl(12);  // output quality
    mylf.lfPitchAndFamily    =           hb_parnl(13);  // pitch and family
    strcpy( mylf.lfFaceName, hb_parc(14) );   // typeface name
    
}


HB_FUNC ( CREATEFONT )
{
    int   nHeight            =           hb_parni(1);   // height of font
    int   nWidth             =           hb_parni(2);   // average character width
    int   nEscapement        =           hb_parni(3);   // angle of escapement
    int   nOrientation       =           hb_parni(4);   // base-line orientation angle
    int   fnWeight           =           hb_parni(5);   // font weight
    DWORD fdwItalic          =           hb_parl(6);    // italic attribute option
    DWORD fdwUnderline       =           hb_parl(7);    // underline attribute option
    DWORD fdwStrikeOut       =           hb_parl(8);    // strikeout attribute option
    DWORD fdwCharSet         =           hb_parnd(9);   // character set identifier
    DWORD fdwOutputPrecision =           hb_parnd(10);  // output precision
    DWORD fdwClipPrecision   =           hb_parnd(11);  // clipping precision
    DWORD fdwQuality         =           hb_parnd(12);  // output quality
    DWORD fdwPitchAndFamily  =           hb_parnd(13);  // pitch and family
    CHAR  *lpszFace          =           hb_parc(14);   // typeface name
    
    hb_retnl( (long) CreateFont( nHeight, nWidth, nEscapement, nOrientation, fnWeight, 
                                 fdwItalic, fdwUnderline, fdwStrikeOut, fdwCharSet, 
                                 fdwOutputPrecision, fdwClipPrecision, fdwQuality, 
                                 fdwPitchAndFamily, lpszFace ) );
                                 
}


HB_FUNC ( CHOOSEFONT ) 
{ 
    CHOOSEFONT cf; 
    LOGFONT lf; 
    HFONT hfont; 
    
    PHB_ITEM pObject  = hb_param( 1, HB_IT_OBJECT );
 
    // Initialize members of the CHOOSEFONT structure. 
 
    cf.lStructSize = sizeof(CHOOSEFONT); 
    cf.hwndOwner = (HWND)NULL; 
    cf.hDC = (HDC)NULL; 
    cf.lpLogFont = &lf; 
    cf.iPointSize = 0; 
    cf.Flags = CF_SCREENFONTS; 
    cf.rgbColors = RGB(0,0,0); 
    cf.lCustData = 0L; 
    cf.lpfnHook = (LPCFHOOKPROC)NULL; 
    cf.lpTemplateName = (LPSTR)NULL; 
    cf.hInstance = (HINSTANCE) NULL; 
    cf.lpszStyle = (LPSTR)NULL; 
    cf.nFontType = SCREEN_FONTTYPE; 
    cf.nSizeMin = 0; 
    cf.nSizeMax = 0; 
 
    // Display the CHOOSEFONT common-dialog box. 
 
    if ( ChooseFont(&cf) ) 
    {
       // Create a logical font based on the user's 
       // selection and return a handle identifying 
       // that font. 
       hfont = CreateFontIndirect(cf.lpLogFont);
       
       //MessageBox( NULL, "Fin qua sono arrivato", "Error", MB_OK | MB_ICONINFORMATION );
       
       if ( hb_pcount()>=1 )
       {
       
         // Fill TFont object
         // nHandle
         hb_vmPushSymbol( hb_dynsymFind( "_NHANDLE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( (long) hfont );
         hb_vmSend(1);
         // nHeight
         hb_vmPushSymbol( hb_dynsymFind( "_NHEIGHT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfHeight );
         hb_vmSend(1);
         // nWidth
         hb_vmPushSymbol( hb_dynsymFind( "_NWIDTH" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfWidth );
         hb_vmSend(1);
         // nEscapement
         hb_vmPushSymbol( hb_dynsymFind( "_NESCAPEMENT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfEscapement );
         hb_vmSend(1);
         // nOrientation
         hb_vmPushSymbol( hb_dynsymFind( "_NORIENTATION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfOrientation );
         hb_vmSend(1);
         // nWeight
         hb_vmPushSymbol( hb_dynsymFind( "_NWEIGHT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfWeight );
         hb_vmSend(1);
         // lItalic
         hb_vmPushSymbol( hb_dynsymFind( "_LITALIC" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfItalic );
         hb_vmSend(1);
         // lUnderline
         hb_vmPushSymbol( hb_dynsymFind( "_LUNDERLINE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfUnderline );
         hb_vmSend(1);
         // lStrikeOut
         hb_vmPushSymbol( hb_dynsymFind( "_LSTRIKEOUT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfStrikeOut );
         hb_vmSend(1);
         // nCharSet
         hb_vmPushSymbol( hb_dynsymFind( "_NCHARSET" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfCharSet );
         hb_vmSend(1);
         // nOutputPrecision
         hb_vmPushSymbol( hb_dynsymFind( "_NOUTPUTPRECISION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfOutPrecision );
         hb_vmSend(1);
         // nClipPrecision
         hb_vmPushSymbol( hb_dynsymFind( "_NCLIPPRECISION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfClipPrecision );
         hb_vmSend(1);
         // nQuality
         hb_vmPushSymbol( hb_dynsymFind( "_NQUALITY" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfQuality );
         hb_vmSend(1);
         // nPitchAndFamily
         hb_vmPushSymbol( hb_dynsymFind( "_NPITCHANDFAMILY" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfPitchAndFamily );
         hb_vmSend(1);
         // cFace
         hb_vmPushSymbol( hb_dynsymFind( "_CFACE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushString( lf.lfFaceName, strlen(lf.lfFaceName) );
         hb_vmSend(1);
       }
    }
    else
       hfont = 0;
       
    hb_retnl( (long) hfont ); 
} 

HB_FUNC ( GETREALFONTVALUE ) 
{ 
    LOGFONT lf; 
    HFONT hfont;
    
    PHB_ITEM pObject  = hb_param( 1, HB_IT_OBJECT );

    memcpy(&lf,&mylf,sizeof(LOGFONT));

    hfont = CreateFontIndirect(&mylf);
 
       //MessageBox( NULL, "Fin qua sono arrivato", "Error", MB_OK | MB_ICONINFORMATION );
       
       if ( hb_pcount()>=1 )
       {
       
         // Fill TFont object
         // nHandle
         hb_vmPushSymbol( hb_dynsymFind( "_NHANDLE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( (long) hfont );
         hb_vmSend(1);
         // nHeight
         hb_vmPushSymbol( hb_dynsymFind( "_NHEIGHT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfHeight );
         hb_vmSend(1);
         // nWidth
         hb_vmPushSymbol( hb_dynsymFind( "_NWIDTH" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfWidth );
         hb_vmSend(1);
         // nEscapement
         hb_vmPushSymbol( hb_dynsymFind( "_NESCAPEMENT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfEscapement );
         hb_vmSend(1);
         // nOrientation
         hb_vmPushSymbol( hb_dynsymFind( "_NORIENTATION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfOrientation );
         hb_vmSend(1);
         // nWeight
         hb_vmPushSymbol( hb_dynsymFind( "_NWEIGHT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfWeight );
         hb_vmSend(1);
         // lItalic
         hb_vmPushSymbol( hb_dynsymFind( "_LITALIC" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfItalic );
         hb_vmSend(1);
         // lUnderline
         hb_vmPushSymbol( hb_dynsymFind( "_LUNDERLINE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfUnderline );
         hb_vmSend(1);
         // lStrikeOut
         hb_vmPushSymbol( hb_dynsymFind( "_LSTRIKEOUT" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLogical( lf.lfStrikeOut );
         hb_vmSend(1);
         // nCharSet
         hb_vmPushSymbol( hb_dynsymFind( "_NCHARSET" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfCharSet );
         hb_vmSend(1);
         // nOutputPrecision
         hb_vmPushSymbol( hb_dynsymFind( "_NOUTPUTPRECISION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfOutPrecision );
         hb_vmSend(1);
         // nClipPrecision
         hb_vmPushSymbol( hb_dynsymFind( "_NCLIPPRECISION" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfClipPrecision );
         hb_vmSend(1);
         // nQuality
         hb_vmPushSymbol( hb_dynsymFind( "_NQUALITY" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfQuality );
         hb_vmSend(1);
         // nPitchAndFamily
         hb_vmPushSymbol( hb_dynsymFind( "_NPITCHANDFAMILY" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushLong( lf.lfPitchAndFamily );
         hb_vmSend(1);
         // cFace
         hb_vmPushSymbol( hb_dynsymFind( "_CFACE" )->pSymbol );
         hb_vmPush( pObject );
         hb_vmPushString( lf.lfFaceName, strlen(lf.lfFaceName) );
         hb_vmSend(1);
       }

    hb_retnl( (long) hfont );

} 


HB_FUNC ( CONVERTPOINTSTOLU )
{
  HDC  hDC;
  int  nPointSize = hb_parni(1); // Font size in points
  long lfHeight;

  hDC = GetDC ( HWND_DESKTOP ) ;
  
  lfHeight = -MulDiv( nPointSize, GetDeviceCaps(hDC, LOGPIXELSY), 72);
  
  ReleaseDC ( HWND_DESKTOP, hDC ) ;
  
  hb_retnl( lfHeight );
  
}
