/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
* C window low level Windows API interface and functions
*
*/
#define _WIN32_WINNT 0x0400
#define WINVER 0x0400
#define _WIN32_IE 0x0501

#include <shlobj.h>

#include <windows.h>
#include <commctrl.h>
#include "hbapi.h"
#include "hbvm.h"
#include "hbstack.h"
#include "hbapiitm.h"

#include "woopgui.h"

// Global Variables:

static HINSTANCE hInst;      // current instance
static char *szAppName = "PROVA";      // Name of the application
//static char *cClassName;     // Name of the application
LPINITCOMMONCONTROLSEX lpInitCtrls;  // Dichiarazione per l'inizializzazione dei controlli

// Foward declarations of functions included in this code module:
LRESULT CALLBACK W_WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK W_DlgProc(HWND, UINT, WPARAM, LPARAM);
void error(LPCTSTR lpszFunction, LPCTSTR dove);
void warning(LPCTSTR lpszFunction, LPCTSTR dove);

/*

//
// RegisterClassEx( cClassName, cIcon, nBkColor, cIconSm ) --> nInstance
//
HB_FUNC ( REGISTERCLASSEX )
{
    WNDCLASSEX wcex;
    HANDLE hInstance = GetModuleHandle( NULL );
    LPCTSTR cClassName;
    char *cIcon, *cIconSm, *cWndProc;
    long nBkColor;
    WNDPROC nlpWinProc;

    hInst = hInstance;

    cClassName         = ( hb_pcount()>0 && !ISNIL(1) ) ? hb_parc(1) : "WoopGUIWinClass" ;
    cIcon              = ( hb_pcount()>1 && !ISNIL(2) ) ? hb_parc(2) : "" ;
    nBkColor           = ( hb_pcount()>2 && !ISNIL(3) ) ? hb_parnl(3) : COLOR_BTNFACE; //COLOR_WINDOW ;
    cIconSm            = ( hb_pcount()>3 && !ISNIL(4) ) ? hb_parc(4) : "" ;
    cWndProc           = ( hb_pcount()>4 && !ISNIL(5) ) ? hb_parc(5) : "W_WndProc" ;

    nlpWinProc         = ( cWndProc == "W_DlgProc" ) ? W_DlgProc : W_WndProc ;

    if (szAppName == NULL)
    {
       lstrcpy (szAppName, cClassName);
    }

    wcex.cbSize        = sizeof(WNDCLASSEX);
    wcex.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wcex.lpfnWndProc   = nlpWinProc;
    wcex.cbClsExtra    = 0;
    wcex.cbWndExtra    = 0;
    wcex.hInstance     = hInstance;
    wcex.hIcon         = ( cIcon == "" ? LoadIcon(NULL, IDI_APPLICATION) : LoadIcon (hInstance, cIcon ) );
    wcex.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground = (HBRUSH)( nBkColor + 1 );
    wcex.lpszMenuName  = NULL;
    wcex.lpszClassName = cClassName;
    wcex.hIconSm       = ( cIconSm == "" ? LoadIcon(NULL, IDI_APPLICATION) : LoadIcon (hInstance, cIconSm ) );

    if ( ! RegisterClassEx(&wcex) )
    {
       MessageBox(0, "Window Registration Failed!", "Error!",
       MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
    return;
    }

    hb_retnl( (LONG) hInstance );
}
*/

//
// RegisterClassEx( cClassName, cIcon, nBkColor, cIconSm ) --> nInstance
//
HB_FUNC ( REGISTERCLASSEX )
{
    WNDCLASSEX wcex;

    UINT      nStyle        = (UINT)      hb_parnl(1);    // Specifies the class style(s). This member can be any combination of the class styles.
    WNDPROC   cWndProc      = W_WndProc; //(WNDPROC)   hb_parnl(2);    // Pointer to the window procedure
    //HINSTANCE hInstance     = (HINSTANCE) hb_parnl(3);    // Handle to the instance that contains the window procedure for the class
    HICON     hIcon         = (HICON)     hb_parnl(4);    // Handle to the class icon. This member must be a handle to an icon resource. If this member is NULL, the system provides a default icon
    HCURSOR   hCursor       = (HCURSOR)   hb_parnl(5);    // Handle to the class cursor. This member must be a handle to a cursor resource. If this member is NULL, an application must explicitly set the cursor shape whenever the mouse moves into the application's window.
    int       hbrBackground =             hb_parni(6);    // Handle to the class background brush
    LPCTSTR   cMenuName     = (LPCTSTR)   hb_parc(7);     // Pointer to a null-terminated character string that specifies the resource name of the class menu, as the name appears in the resource file. If you use an integer to identify the menu, use the MAKEINTRESOURCE macro. If this member is NULL, windows belonging to this class have no default menu
    LPCTSTR   cClassName    = (LPCTSTR)   hb_parc(8);     // Pointer to a null-terminated string
    HICON     hIconSm       = (HICON)     hb_parnl(9);    // Handle to a small icon that is associated with the window class. If this member is NULL, the system searches the icon resource specified by the hIcon member for an icon of the appropriate size to use as the small icon

    HINSTANCE hInstance = GetModuleHandle( NULL );

    wcex.cbSize        = sizeof(WNDCLASSEX);
    wcex.style         = nStyle;
    wcex.lpfnWndProc   = cWndProc;
    wcex.cbClsExtra    = 0;
    wcex.cbWndExtra    = 0;
    wcex.hInstance     = hInstance;
    wcex.hIcon         = hIcon;
    wcex.hCursor       = hCursor;
    wcex.hbrBackground = (HBRUSH) (hbrBackground+1);
    wcex.lpszMenuName  = cMenuName;
    wcex.lpszClassName = cClassName;
    wcex.hIconSm       = hIconSm;

    hInst = hInstance;

    if ( ! RegisterClassEx(&wcex) )
    {
       error("Window Registration Failed!", "RegisterClassEx");
       //MessageBox(0, "Window Registration Failed!", "Error!",
       //MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
    return;
    }

    hb_retnl( (LONG) hInstance );
}


//
// UnRegisterClassEx( cClassName, nHinstance ) --> lOk
//
HB_FUNC ( UNREGISTERCLASS )
{

    LPCTSTR cClassName = (LPCTSTR) hb_parc(1);
    HANDLE hInstance   = (HANDLE)  hb_parnl(2);
    BOOL   lOk ;

    if ( ! ( lOk = UnregisterClass(cClassName,hInstance ) ) )
    {
       MessageBox(0, "Window Registration Failed!", "Error!",
       MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
    return;
    }

    hb_retl( (BOOL) lOk );
}


// The BringWindowToTop function brings the specified window to the top of the Z order.
// If the window is a top-level window, it is activated. If the window is a child window,
// the top-level parent window associated with the child window is activated.
//
HB_FUNC ( BRINGWINDOWTOTOP )
{
    HWND hWnd = (HWND) hb_parnl (1);         // handle to window

    hb_retl( (BOOL) BringWindowToTop( hWnd ) );
}


//void DefineInstance( HINSTANCE hInstance )
//{
//   //hInst = hInstance;
//}

// Creates an overlapped, pop-up, or child window with an extended window style.
//
HB_FUNC ( CREATEWINDOWEX )
{
    //CLIENTCREATESTRUCT ccs;
                           // Bisogna definire i defaults
    HWND hwnd;
    DWORD nExStyle       = hb_parnd(1);
    LPCTSTR lcClassName  = (!ISNIL(2) ? (LPCTSTR) hb_parc(2) : szAppName );
    LPCTSTR cName        = (LPCTSTR) hb_parc(3);
    DWORD nStyle         = (DWORD) hb_parnd(4);
    int   nRow           = hb_parni(5);
    int   nCol           = hb_parni(6);
    int   nWidth         = hb_parni(7);
    int   nHeight        = hb_parni(8);
    HWND  nParent        = (HWND)  hb_parnl(9);
    HMENU nMenu          = (HMENU) hb_parnl(10);
    HINSTANCE hInstance  = hInst;
    LPVOID pStruct       = NULL;   // da sistemare

    HB_TRACE(HB_TR_DEBUG, ("CreateWindowsEx()"));

    //warning("Prima della creazione della finestra", "CreateWindowEx" );

    hwnd = CreateWindowEx( nExStyle,
                           lcClassName,
                           cName,
                           nStyle,
                           nRow,
                           nCol,
                           nWidth,
                           nHeight,
                           nParent,
                           nMenu,
                           hInstance,
                           pStruct);

    //warning("Dopo la creazione della finestra", "CreateWindowEx" );

    if(hwnd == NULL)
    {
      error( (LPCTSTR) "Window Creation", (LPCTSTR) cName );
      //MessageBox(0, "Window Creation Failed!", "Error!",
      //MB_ICONEXCLAMATION | MB_OK | MB_SYSTEMMODAL);
    return;
    }

    hb_retnl ((LONG)hwnd);
}

// Funzioni di Window

/*
HB_FUNC ( ADJUSTWINDOWRECTEX )
{
    LPRECT lpRect =

    hb_retl( (BOOL) AdjustWindowRectEx(
                                        LPRECT lpRect,    // client-rectangle structure
                                        DWORD dwStyle,    // window styles
                                        BOOL bMenu,       // menu-present option
                                        DWORD dwExStyle   // extended window style
                                      )
           );

}
*/

// From MiniGUI - Roberto Lopez
//
HB_FUNC ( CENTERWINDOW )
{
    RECT rect;
    int w, h, x, y;
    HWND hWnd = (HWND) hb_parnl (1);

    GetWindowRect(hWnd, &rect);

    w = rect.right  - rect.left;
    h = rect.bottom - rect.top;
    x = GetSystemMetrics(SM_CXSCREEN);
    y = GetSystemMetrics(SM_CYSCREEN);
    SetWindowPos(hWnd, HWND_TOP, (x - w) / 2, (y - h) / 2, 0, 0, SWP_NOSIZE);
}

// Minimizes (but does not destroy) the specified window.
//
HB_FUNC ( CLOSEWINDOW )
{
    HWND hWnd = (HWND) hb_parnl (1);
    CloseWindow(hWnd);
}

// The DestroyWindow function destroys the specified window.
//
HB_FUNC ( DESTROYWINDOW )
{
    HWND hWnd = (HWND) hb_parnl (1);
    DestroyWindow(hWnd);
}

HB_FUNC ( ENABLEWINDOW )
{
    HWND hwnd          = (HWND) hb_parnl(1);    // handle to window
    BOOL bEnable       = hb_parl(2);            // enable or disable input

    hb_retl( (BOOL) EnableWindow( hwnd, bEnable ) );
}

// The FindWindow function retrieves a handle to the top-level window whose class name and
// window name match the specified strings. This function does not search child windows.
// This function does not perform a case-sensitive search.
//
HB_FUNC ( FINDWINDOW )
{
    LPCTSTR lpszClass   = (LPCTSTR) hb_parc(1);  // class name
    LPCTSTR lpszWindow  = (LPCTSTR) hb_parc(2);  // window name

    hb_retnl( (long) FindWindow( lpszClass, lpszWindow ) );
}

// The FindWindowEx function retrieves a handle to a window whose class name and window name
// match the specified strings. The function searches child windows, beginning with the one
// following the specified child window. This function does not perform a case-sensitive search.
//
HB_FUNC ( FINDWINDOWEX )
{
    HWND hwndParent     = (HWND) hb_parnl(1);    // handle to parent window
    HWND hwndChildAfter = (HWND) hb_parnl(2);    // handle to child window
    LPCTSTR lpszClass   = (LPCTSTR) hb_parc(3);  // class name
    LPCTSTR lpszWindow  = (LPCTSTR) hb_parc(4);  // window name

    hb_retnl( (long) FindWindowEx( hwndParent, hwndChildAfter, lpszClass, lpszWindow ) );
}

// The GetFocus function retrieves the handle to the window that has the keyboard focus,
// if the window is attached to the calling thread's message queue.
//
HB_FUNC ( GETFOCUS )
{
    hb_retnl( (long) GetFocus() );      // Return the handle of current window or control focused
}

// The GetParent function retrieves a handle to the specified window's parent or owner.
//
HB_FUNC ( GETPARENT )
{
    HWND hwndChild     = (HWND) hb_parnl(1);    // handle of child window

    hb_retnl( (long) GetParent( hwndChild ) );
}

// The GetWindowText function copies the text of the specified window's title bar (if it has one)
// into a buffer. If the specified window is a control, the text of the control is copied.
// However, GetWindowText cannot retrieve the text of a control in another application.
//
HB_FUNC ( GETWINDOWTEXT )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control
    LPCTSTR lpString    = (LPCTSTR) hb_parc(2);  // text buffer
    int nMaxCount       =  hb_parni(3);          // maximum number of characters to copy

    hb_retni( (int) GetWindowText( hwnd, &lpString, nMaxCount ) );
    hb_storc( lpString, 2 );
}

// The IsChild function tests whether a window is a child window or descendant window of a specified
// parent window.
//
HB_FUNC ( ISCHILD )
{
    HWND hwndParent     = (HWND) hb_parnl(1);    // handle to parent window
    HWND hwnd           = (HWND) hb_parnl(2);    // handle of window to test

    hb_retl( (BOOL) IsChild( hwndParent, hwnd ) );
}

// The IsIconic function determines whether the specified window is minimized (iconic).
//
HB_FUNC ( ISICONIC )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle of window to test

    hb_retl( (BOOL) IsIconic( hwnd ) );
}

// The IsWindow function determines whether the specified window handle identifies an
// existing window.
//
HB_FUNC ( ISWINDOW )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle of window to test

    hb_retl( (BOOL) IsWindow( hwnd ) );
}

// The IsWindow function determines whether the specified window handle identifies an
// existing window.
//
HB_FUNC ( ISWINDOWVISIBLE )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle of window to test

    hb_retl( (BOOL) IsWindowVisible( hwnd ) );
}

// The IsZoomed function determines whether a window is maximized.
//
HB_FUNC ( ISZOOMED )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle of window to test

    hb_retl( (BOOL) IsZoomed( hwnd ) );
}

// The MoveWindow function changes the position and dimensions of the specified window.
// For a top-level window, the position and dimensions are relative to the upper-left corner of
// the screen. For a child window, they are relative to the upper-left corner of the parent
// window's client area.
//
HB_FUNC ( MOVEWINDOW )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window
    int  X              = hb_parni(2);           // horizontal position
    int  Y              = hb_parni(3);           // vertical position
    int  nWidth         = hb_parni(4);           // width
    int  nHeight        = hb_parni(5);           // height
    BOOL bRepaint       = hb_parl(6);            // repaint option

    hb_retl( (BOOL) MoveWindow( hwnd, X, Y, nWidth, nHeight, bRepaint ) );
}

// The OpenIcon function restores a minimized (iconic) window to its previous size and position;
// it then activates the window.
//
HB_FUNC ( OPENICON )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window

    hb_retl( (BOOL) OpenIcon( hwnd ) );
}

// The SetFocus function sets the keyboard focus to the specified window. The window must be
// attached to the calling thread's message queue.
//
HB_FUNC ( SETFOCUS )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control

    hb_retnl( (long) SetFocus( hwnd ) );      // Return the handle of previous window or control focused
}

// The SetParent function changes the parent window of the specified child window. 
//
HB_FUNC ( SETPARENT )
{
    HWND hwndChild     = (HWND) hb_parnl(1);    // handle of child window
    HWND hwndNewParent = (HWND) hb_parnl(2);    // handle of new parent window
    
    HWND hwndOldParent = SetParent( hwndChild, hwndNewParent );    // handle of old parent window

    hb_retnl( (long) hwndOldParent );
}

// The SetWindowText function changes the text of the specified window's title bar (if it has one).
// If the specified window is a control, the text of the control is changed.
// However, SetWindowText cannot change the text of a control in another application.
//
HB_FUNC ( SETWINDOWTEXT )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control
    LPCTSTR lpString    = (LPCTSTR) hb_parc(2);  // title or text

    hb_retl( (BOOL) SetWindowText( hwnd, lpString ) );
}

// The SetWindowLongPtr function changes an attribute of the specified window.
// The function also sets a value at the specified offset in the extra window memory.
//
HB_FUNC ( SETWINDOWLONGPTR )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control
    int  nIndex         = hb_parni(2);           // offset of value to set
    LONG_PTR dwNewLong  = (LONG_PTR) hb_parnl(3); // new value

    hb_retnl( (LONG_PTR) SetWindowLongPtr( hwnd, nIndex, dwNewLong ) );
}

// The SetWindowPos function changes the size, position, and Z order of a child, pop-up, or
// top-level window. Child, pop-up, and top-level windows are ordered according to their
// appearance on the screen. The topmost window receives the highest rank and is the first
// window in the Z order.
//
HB_FUNC ( SETWINDOWPOS )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control
    HWND hWndInsertAfter= (HWND) hb_parnl(2);    // placement-order handle
    int X               =        hb_parni(3);    // horizontal position
    int Y               =        hb_parni(4);    // vertical position
    int cx              =        hb_parni(5);    // width
    int cy              =        hb_parni(6);    // height
    UINT uFlags         = (UINT) hb_parni(7);    // window-positioning options

    hb_retl( (BOOL) SetWindowPos( hwnd, hWndInsertAfter, X, Y, cx, cy, uFlags ) );
}

// The ShowWindow function sets the specified window's show state.
//
HB_FUNC ( SHOWWINDOW )
{
    HWND hWnd         = (HWND) hb_parnl (1);    // handle to window
    int  nCmdShow     = hb_parni (2);           // show state

    hb_retl( (BOOL) ShowWindow(hWnd, nCmdShow) );
}

// The UpdateWindow function updates the client area of the specified window by sending
// a WM_PAINT message to the window if the window's update region is not empty.
// The function sends a WM_PAINT message directly to the window procedure of the specified
// window, bypassing the application queue. If the update region is empty, no message is sent.
//
HB_FUNC ( UPDATEWINDOW )
{
    HWND hWnd         = (HWND) hb_parnl (1);    // handle to window

    hb_retl( (BOOL) UpdateWindow(hWnd) );
}

//
// Interfaccia per la gestione degli eventi con le api di windows
// Questa procedura richiama la funzione DEFWNDPROC (se definita) passando l'evento
// da controllare.
// Se non viene fatto nulla passa alla gestione di default
//

LRESULT CALLBACK W_WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static PHB_DYNS Dyns = 0 ;
    long int res;

    if ( ! Dyns )
    {
        Dyns = hb_dynsymFind( "W_DEFWNDEVENTS" );
    }

    hb_vmPushSymbol( Dyns->pSymbol );
    hb_vmPushNil();
    hb_vmPushLong( ( LONG ) hWnd );
    hb_vmPushLong( message );
    hb_vmPushLong( wParam );
    hb_vmPushLong( lParam );
    hb_vmDo( 4 );

    res = hb_itemGetNL( (PHB_ITEM) &hb_stack.Return );
    if( res == -1 )
    {
        return DefWindowProc( ( HWND ) hWnd, message, wParam, lParam );
        //return DefDlgProc( ( HWND ) hWnd, message, wParam, lParam );
    }
    else
    {
        return res;
    }
}

LRESULT CALLBACK W_DlgProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static PHB_DYNS Dyns = 0 ;
    long int res;

    if ( ! Dyns )
    {
        Dyns = hb_dynsymFind( "W_DLGEVENTS" );
    }

    hb_vmPushSymbol( Dyns->pSymbol );
    hb_vmPushNil();
    hb_vmPushLong( ( LONG ) hWnd );
    hb_vmPushLong( message );
    hb_vmPushLong( wParam );
    hb_vmPushLong( lParam );
    hb_vmDo( 4 );

    res = hb_itemGetNL( (PHB_ITEM) &hb_stack.Return );
    if( res == -1 )
    {
        return DefDlgProc( ( HWND ) hWnd, message, wParam, lParam );
    }
    else
    {
        return res;
    }
}


HB_FUNC( POSTQUITMESSAGE )
{
    PostQuitMessage( hb_parnl( 1 ) );
}

HB_FUNC ( SENDMESSAGE )
{
    hb_retnl( (LONG) SendMessage(
                       (HWND) hb_parnl( 1 ),    // handle of destination window
                       (UINT) hb_parni( 2 ),    // message to send
                       (WPARAM) hb_parnl( 3 ),  // first message parameter
                       (LPARAM) hb_parnl( 4 )   // second message parameter
                     ) );
}

HB_FUNC ( POSTMESSAGE )
{
    hb_retnl( (LONG) PostMessage(
                       (HWND) hb_parnl( 1 ),    // handle of destination window
                       (UINT) hb_parni( 2 ),    // message to send
                       (WPARAM) hb_parnl( 3 ),  // first message parameter
                       (LPARAM) hb_parnl( 4 )   // second message parameter
                     ) );
}

// Restituisce
HB_FUNC ( GETWINDOWLONG )
{
    hb_retnl( (LONG) GetWindowLong(
                            (HWND) hb_parnl( 1 ),   // handle of destination window
                            hb_parni(2)             // offset of value to retrieve
                                       )
            );
}

// The GetWindowLongPtr function retrieves information about the specified window.
//The function also retrieves the value at a specified offset into the extra window memory.
//
HB_FUNC ( GETWINDOWLONGPTR )
{
    HWND hwnd           = (HWND) hb_parnl(1);    // handle to window or control
    int  nIndex         = hb_parni(2);           // offset of value to retrieve

    hb_retnl( (LONG_PTR) GetWindowLongPtr( hwnd, nIndex ) );
}



// Inizializza i controlli dell'applicazione
HB_FUNC( INITCOMMONCONTROLSEX )
{
  hb_retl( (BOOL) InitCommonControlsEx( (LPINITCOMMONCONTROLSEX) lpInitCtrls ) );

}

// Funzione per la visualizzazione dell'errore
void error(LPCTSTR lpszFunction, LPCTSTR dove)
{
    LPVOID lpMsgBuf;
    char   cErrore[200];
    DWORD dw = GetLastError();
    static PHB_DYNS Dyns = 0 ;
    long int res;

    FormatMessage(
                   FORMAT_MESSAGE_ALLOCATE_BUFFER |
                   FORMAT_MESSAGE_FROM_SYSTEM |
                   FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL,
                   dw,
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                   (LPTSTR) &lpMsgBuf,
                   0,
                   NULL
                 );

    sprintf(cErrore, "%s failed %s.\n Codice %u.\n %s\n",
        lpszFunction, dove, dw, (LPCTSTR)lpMsgBuf);

    // Display the string.
    MessageBox( NULL, (LPCTSTR)cErrore, "Error", MB_OK | MB_ICONINFORMATION );
    // Free the buffer.
    LocalFree( lpMsgBuf );

    if ( ! Dyns )
    {
        Dyns = hb_dynsymFind( "GENMYERROR" );
    }

    hb_vmPushSymbol( Dyns->pSymbol );
    hb_vmPushNil();
    hb_vmPushString( cErrore, strlen(cErrore) );
    hb_vmDo( 1 );

    return;
    //res = hb_itemGetNL( (PHB_ITEM) &hb_stack.Return );
    //if( res == -1 )
    //{
    //    //return DefWindowProc( ( HWND ) hWnd, message, wParam, lParam );
    //    return DefDlgProc( ( HWND ) hWnd, message, wParam, lParam );
    //}
    //else
    //{
    //  return res;
    //}

}

// Funzione per la visualizzazione dell'errore
void warning(LPCTSTR lpszFunction, LPCTSTR dove)
{
    LPVOID lpMsgBuf;
    char   cErrore[200];
    DWORD dw = GetLastError();

    FormatMessage(
                   FORMAT_MESSAGE_ALLOCATE_BUFFER |
                   FORMAT_MESSAGE_FROM_SYSTEM |
                   FORMAT_MESSAGE_IGNORE_INSERTS,
                   NULL,
                   dw,
                   MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                   (LPTSTR) &lpMsgBuf,
                   0,
                   NULL
                 );

    sprintf(cErrore, "%s - %s.\n Codice %u.\n %s\n",
        lpszFunction, dove, dw, (LPCTSTR)lpMsgBuf);

    // Display the string.
    MessageBox( NULL, (LPCTSTR)cErrore, "Warning", MB_OK | MB_ICONINFORMATION );
    // Free the buffer.
    LocalFree( lpMsgBuf );

}

HB_FUNC ( MAINLOOP )
{
   MSG msg ;
   BOOL bRet;
   HANDLE hAccelTable;

   hAccelTable = LoadAccelerators (hInst, szAppName);

   // Main message loop:
    while( (bRet = GetMessage( &msg, (HWND) NULL, 0, 0 )) != 0)
    {
        if (bRet == -1)
        {
            // handle the error and possibly exit
        }
        else
        {
         if (!TranslateAccelerator (msg.hwnd, hAccelTable, &msg))
         {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
         }
        }
    }

    // Return the exit code to the system.

    //return msg.wParam;
}

/* CREATE A TOOLTIP CONTROL OVER THE ENTIRE WINDOW AREA */
HB_FUNC( SETTOOLTIP )
{
    HWND hwnd      = (HWND) hb_parnl(1);
    HWND hwndTT    = (HWND) hb_parnl(2);    // handle to the ToolTip control
    char *strTT    = (LPTSTR) hb_parc(3);   // Stringa del tooltip

          // struct specifying info about tool in ToolTip control

                 // struct specifying control classes to register
    INITCOMMONCONTROLSEX iccex;
    TOOLINFO ti;
    unsigned int uid = 0;       // for ti initialization
    LPTSTR lptstr = strTT;
    RECT rect;                  // for client area coordinates

    /* INITIALIZE COMMON CONTROLS */
    iccex.dwICC = ICC_WIN95_CLASSES;
    iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    InitCommonControlsEx(&iccex);

    /* GET COORDINATES OF THE MAIN CLIENT AREA */
    GetClientRect (hwnd, &rect);

    /* INITIALIZE MEMBERS OF THE TOOLINFO STRUCTURE */
    ti.cbSize = sizeof(TOOLINFO);
    ti.uFlags = TTF_SUBCLASS;
    ti.hwnd = hwnd;
    //ti.hinst = ghThisInstance;
    ti.uId = uid;
    ti.lpszText = lptstr;
        // ToolTip control will cover the whole window
    ti.rect.left = rect.left;
    ti.rect.top = rect.top;
    ti.rect.right = rect.right;
    ti.rect.bottom = rect.bottom;

    /* SEND AN ADDTOOL MESSAGE TO THE TOOLTIP CONTROL WINDOW */
    SendMessage(hwndTT, TTM_ADDTOOL, 0, (LPARAM) (LPTOOLINFO) &ti);
}

// DoCreateStatusBar - creates a status bar and divides it into
//     the specified number of parts.
// Returns the handle to the status bar.
// hwndParent - parent window for the status bar.
// nStatusID - child window identifier.
// hinst - handle to the application instance.
// nParts - number of parts into which to divide the status bar.
HB_FUNC( CREATESTATUSBAR )
{
    HWND hwndParent = (HWND) hb_parnl(1);    // handle to the Parent window
    HWND hwndSB     = (HWND) hb_parnl(2);    // handle to the StatusBar control
    int  nParts     = ( !ISNIL(3) ? hb_parni(3) : 1 );

    RECT rcClient;
    HLOCAL hloc;
    LPINT lpParts;
    int i, nWidth;

    // Get the coordinates of the parent window's client area.
    GetClientRect(hwndParent, &rcClient);

    // Allocate an array for holding the right edge coordinates.
    hloc = LocalAlloc(LHND, sizeof(int) * nParts);
    lpParts = LocalLock(hloc);

    // Calculate the right edge coordinate for each part, and
    // copy the coordinates to the array.
    nWidth = rcClient.right / nParts;
    for (i = 0; i < nParts; i++)
    {
        lpParts[i] = nWidth;
        nWidth += nWidth;
    }

    // Tell the status bar to create the window parts.
    SendMessage(hwndSB, SB_SETPARTS, (WPARAM) nParts, (LPARAM) lpParts);

    // Free the array, and return.
    LocalUnlock(hloc);
    LocalFree(hloc);
}


HB_FUNC ( SETSTATUSBAR )
{
    HWND hwnd     = (HWND)  hb_parnl (1);    // handle to the StatusBar control
    int  iPart    =         hb_parni (2)-1;  // part of statusbar to write
    int  uFlags   =         hb_parni (3);    // options flags
    LPSTR szText  = (LPSTR) hb_parc(4);      // text to write

    SendMessage(hwnd, SB_SETTEXT, (WPARAM) iPart | uFlags , (LPARAM) (LPSTR) szText );
}

const char *DecodeMessageName(int message)
{

    static char s_szBuf[128];

    switch ( message )
    {
        case 0x0000: return "WM_NULL";
        case 0x0001: return "WM_CREATE";
        case 0x0002: return "WM_DESTROY";
        case 0x0003: return "WM_MOVE";
        case 0x0005: return "WM_SIZE";
        case 0x0006: return "WM_ACTIVATE";
        case 0x0007: return "WM_SETFOCUS";
        case 0x0008: return "WM_KILLFOCUS";
        case 0x000A: return "WM_ENABLE";
        case 0x000B: return "WM_SETREDRAW";
        case 0x000C: return "WM_SETTEXT";
        case 0x000D: return "WM_GETTEXT";
        case 0x000E: return "WM_GETTEXTLENGTH";
        case 0x000F: return "WM_PAINT";
        case 0x0010: return "WM_CLOSE";
        case 0x0011: return "WM_QUERYENDSESSION";
        case 0x0012: return "WM_QUIT";
        case 0x0013: return "WM_QUERYOPEN";
        case 0x0014: return "WM_ERASEBKGND";
        case 0x0015: return "WM_SYSCOLORCHANGE";
        case 0x0016: return "WM_ENDSESSION";
        case 0x0017: return "WM_SYSTEMERROR";
        case 0x0018: return "WM_SHOWWINDOW";
        case 0x0019: return "WM_CTLCOLOR";
        case 0x001A: return "WM_WININICHANGE";
        case 0x001B: return "WM_DEVMODECHANGE";
        case 0x001C: return "WM_ACTIVATEAPP";
        case 0x001D: return "WM_FONTCHANGE";
        case 0x001E: return "WM_TIMECHANGE";
        case 0x001F: return "WM_CANCELMODE";
        case 0x0020: return "WM_SETCURSOR";
        case 0x0021: return "WM_MOUSEACTIVATE";
        case 0x0022: return "WM_CHILDACTIVATE";
        case 0x0023: return "WM_QUEUESYNC";
        case 0x0024: return "WM_GETMINMAXINFO";
        case 0x0026: return "WM_PAINTICON";
        case 0x0027: return "WM_ICONERASEBKGND";
        case 0x0028: return "WM_NEXTDLGCTL";
        case 0x002A: return "WM_SPOOLERSTATUS";
        case 0x002B: return "WM_DRAWITEM";
        case 0x002C: return "WM_MEASUREITEM";
        case 0x002D: return "WM_DELETEITEM";
        case 0x002E: return "WM_VKEYTOITEM";
        case 0x002F: return "WM_CHARTOITEM";
        case 0x0030: return "WM_SETFONT";
        case 0x0031: return "WM_GETFONT";
        case 0x0037: return "WM_QUERYDRAGICON";
        case 0x0039: return "WM_COMPAREITEM";
        case 0x0041: return "WM_COMPACTING";
        case 0x0044: return "WM_COMMNOTIFY";
        case 0x0046: return "WM_WINDOWPOSCHANGING";
        case 0x0047: return "WM_WINDOWPOSCHANGED";
        case 0x0048: return "WM_POWER";

#ifdef  __WIN32__
        case 0x004A: return "WM_COPYDATA";
        case 0x004B: return "WM_CANCELJOURNAL";
        case 0x004E: return "WM_NOTIFY";
        case 0x0050: return "WM_INPUTLANGCHANGEREQUEST";
        case 0x0051: return "WM_INPUTLANGCHANGE";
        case 0x0052: return "WM_TCARD";
        case 0x0053: return "WM_HELP";
        case 0x0054: return "WM_USERCHANGED";
        case 0x0055: return "WM_NOTIFYFORMAT";
        case 0x007B: return "WM_CONTEXTMENU";
        case 0x007C: return "WM_STYLECHANGING";
        case 0x007D: return "WM_STYLECHANGED";
        case 0x007E: return "WM_DISPLAYCHANGE";
        case 0x007F: return "WM_GETICON";
        case 0x0080: return "WM_SETICON";
#endif  //WIN32

        case 0x0081: return "WM_NCCREATE";
        case 0x0082: return "WM_NCDESTROY";
        case 0x0083: return "WM_NCCALCSIZE";
        case 0x0084: return "WM_NCHITTEST";
        case 0x0085: return "WM_NCPAINT";
        case 0x0086: return "WM_NCACTIVATE";
        case 0x0087: return "WM_GETDLGCODE";
        case 0x00A0: return "WM_NCMOUSEMOVE";
        case 0x00A1: return "WM_NCLBUTTONDOWN";
        case 0x00A2: return "WM_NCLBUTTONUP";
        case 0x00A3: return "WM_NCLBUTTONDBLCLK";
        case 0x00A4: return "WM_NCRBUTTONDOWN";
        case 0x00A5: return "WM_NCRBUTTONUP";
        case 0x00A6: return "WM_NCRBUTTONDBLCLK";
        case 0x00A7: return "WM_NCMBUTTONDOWN";
        case 0x00A8: return "WM_NCMBUTTONUP";
        case 0x00A9: return "WM_NCMBUTTONDBLCLK";
        case 0x0100: return "WM_KEYDOWN";
        case 0x0101: return "WM_KEYUP";
        case 0x0102: return "WM_CHAR";
        case 0x0103: return "WM_DEADCHAR";
        case 0x0104: return "WM_SYSKEYDOWN";
        case 0x0105: return "WM_SYSKEYUP";
        case 0x0106: return "WM_SYSCHAR";
        case 0x0107: return "WM_SYSDEADCHAR";
        case 0x0108: return "WM_KEYLAST";

#ifdef  __WIN32__
        case 0x010D: return "WM_IME_STARTCOMPOSITION";
        case 0x010E: return "WM_IME_ENDCOMPOSITION";
        case 0x010F: return "WM_IME_COMPOSITION";
#endif  //WIN32

        case 0x0110: return "WM_INITDIALOG";
        case 0x0111: return "WM_COMMAND";
        case 0x0112: return "WM_SYSCOMMAND";
        case 0x0113: return "WM_TIMER";
        case 0x0114: return "WM_HSCROLL";
        case 0x0115: return "WM_VSCROLL";
        case 0x0116: return "WM_INITMENU";
        case 0x0117: return "WM_INITMENUPOPUP";
        case 0x011F: return "WM_MENUSELECT";
        case 0x0120: return "WM_MENUCHAR";
        case 0x0121: return "WM_ENTERIDLE";

        // #if(WINVER >= 0x0500)
        // #ifndef _WIN32_WCE
        case 0x0122: return "WM_MENURBUTTONUP";
        case 0x0123: return "WM_MENUDRAG";
        case 0x0124: return "WM_MENUGETOBJECT";
        case 0x0125: return "WM_UNINITMENUPOPUP";
        case 0x0126: return "WM_MENUCOMMAND";

        // #ifndef _WIN32_WCE
        // #if(_WIN32_WINNT >= 0x0500)
        case 0x0127: return "WM_CHANGEUISTATE";
        case 0x0128: return "WM_UPDATEUISTATE";
        case 0x0129: return "WM_QUERYUISTATE";

        case 0x0132: return "WM_CTLCOLORMSGBOX";
        case 0x0133: return "WM_CTLCOLOREDIT";
        case 0x0134: return "WM_CTLCOLORLISTBOX";
        case 0x0135: return "WM_CTLCOLORBTN";
        case 0x0136: return "WM_CTLCOLORDLG";
        case 0x0137: return "WM_CTLCOLORSCROLLBAR";
        case 0x0138: return "WM_CTLCOLORSTATIC";

        case 0x0200: return "WM_MOUSEMOVE";
        case 0x0201: return "WM_LBUTTONDOWN";
        case 0x0202: return "WM_LBUTTONUP";
        case 0x0203: return "WM_LBUTTONDBLCLK";
        case 0x0204: return "WM_RBUTTONDOWN";
        case 0x0205: return "WM_RBUTTONUP";
        case 0x0206: return "WM_RBUTTONDBLCLK";
        case 0x0207: return "WM_MBUTTONDOWN";
        case 0x0208: return "WM_MBUTTONUP";
        case 0x0209: return "WM_MBUTTONDBLCLK";
        case 0x0210: return "WM_PARENTNOTIFY";
        case 0x0211: return "WM_ENTERMENULOOP";
        case 0x0212: return "WM_EXITMENULOOP";

#ifdef  __WIN32__
        case 0x0213: return "WM_NEXTMENU";
        case 0x0214: return "WM_SIZING";
        case 0x0215: return "WM_CAPTURECHANGED";
        case 0x0216: return "WM_MOVING";
        case 0x0218: return "WM_POWERBROADCAST";
        case 0x0219: return "WM_DEVICECHANGE";
#endif  //WIN32

        case 0x0220: return "WM_MDICREATE";
        case 0x0221: return "WM_MDIDESTROY";
        case 0x0222: return "WM_MDIACTIVATE";
        case 0x0223: return "WM_MDIRESTORE";
        case 0x0224: return "WM_MDINEXT";
        case 0x0225: return "WM_MDIMAXIMIZE";
        case 0x0226: return "WM_MDITILE";
        case 0x0227: return "WM_MDICASCADE";
        case 0x0228: return "WM_MDIICONARRANGE";
        case 0x0229: return "WM_MDIGETACTIVE";
        case 0x0230: return "WM_MDISETMENU";
        case 0x0233: return "WM_DROPFILES";

#ifdef  __WIN32__
        case 0x0281: return "WM_IME_SETCONTEXT";
        case 0x0282: return "WM_IME_NOTIFY";
        case 0x0283: return "WM_IME_CONTROL";
        case 0x0284: return "WM_IME_COMPOSITIONFULL";
        case 0x0285: return "WM_IME_SELECT";
        case 0x0286: return "WM_IME_CHAR";
        case 0x0290: return "WM_IME_KEYDOWN";
        case 0x0291: return "WM_IME_KEYUP";
#endif  //WIN32

        case 0x0300: return "WM_CUT";
        case 0x0301: return "WM_COPY";
        case 0x0302: return "WM_PASTE";
        case 0x0303: return "WM_CLEAR";
        case 0x0304: return "WM_UNDO";
        case 0x0305: return "WM_RENDERFORMAT";
        case 0x0306: return "WM_RENDERALLFORMATS";
        case 0x0307: return "WM_DESTROYCLIPBOARD";
        case 0x0308: return "WM_DRAWCLIPBOARD";
        case 0x0309: return "WM_PAINTCLIPBOARD";
        case 0x030A: return "WM_VSCROLLCLIPBOARD";
        case 0x030B: return "WM_SIZECLIPBOARD";
        case 0x030C: return "WM_ASKCBFORMATNAME";
        case 0x030D: return "WM_CHANGECBCHAIN";
        case 0x030E: return "WM_HSCROLLCLIPBOARD";
        case 0x030F: return "WM_QUERYNEWPALETTE";
        case 0x0310: return "WM_PALETTEISCHANGING";
        case 0x0311: return "WM_PALETTECHANGED";

#ifdef __WIN32__
        // common controls messages - although they're not strictly speaking
        // standard, it's nice to decode them nevertheless

        // listview
        case 0x1000 + 0: return "LVM_GETBKCOLOR";
        case 0x1000 + 1: return "LVM_SETBKCOLOR";
        case 0x1000 + 2: return "LVM_GETIMAGELIST";
        case 0x1000 + 3: return "LVM_SETIMAGELIST";
        case 0x1000 + 4: return "LVM_GETITEMCOUNT";
        case 0x1000 + 5: return "LVM_GETITEMA";
        case 0x1000 + 75: return "LVM_GETITEMW";
        case 0x1000 + 6: return "LVM_SETITEMA";
        case 0x1000 + 76: return "LVM_SETITEMW";
        case 0x1000 + 7: return "LVM_INSERTITEMA";
        case 0x1000 + 77: return "LVM_INSERTITEMW";
        case 0x1000 + 8: return "LVM_DELETEITEM";
        case 0x1000 + 9: return "LVM_DELETEALLITEMS";
        case 0x1000 + 10: return "LVM_GETCALLBACKMASK";
        case 0x1000 + 11: return "LVM_SETCALLBACKMASK";
        case 0x1000 + 12: return "LVM_GETNEXTITEM";
        case 0x1000 + 13: return "LVM_FINDITEMA";
        case 0x1000 + 83: return "LVM_FINDITEMW";
        case 0x1000 + 14: return "LVM_GETITEMRECT";
        case 0x1000 + 15: return "LVM_SETITEMPOSITION";
        case 0x1000 + 16: return "LVM_GETITEMPOSITION";
        case 0x1000 + 17: return "LVM_GETSTRINGWIDTHA";
        case 0x1000 + 87: return "LVM_GETSTRINGWIDTHW";
        case 0x1000 + 18: return "LVM_HITTEST";
        case 0x1000 + 19: return "LVM_ENSUREVISIBLE";
        case 0x1000 + 20: return "LVM_SCROLL";
        case 0x1000 + 21: return "LVM_REDRAWITEMS";
        case 0x1000 + 22: return "LVM_ARRANGE";
        case 0x1000 + 23: return "LVM_EDITLABELA";
        case 0x1000 + 118: return "LVM_EDITLABELW";
        case 0x1000 + 24: return "LVM_GETEDITCONTROL";
        case 0x1000 + 25: return "LVM_GETCOLUMNA";
        case 0x1000 + 95: return "LVM_GETCOLUMNW";
        case 0x1000 + 26: return "LVM_SETCOLUMNA";
        case 0x1000 + 96: return "LVM_SETCOLUMNW";
        case 0x1000 + 27: return "LVM_INSERTCOLUMNA";
        case 0x1000 + 97: return "LVM_INSERTCOLUMNW";
        case 0x1000 + 28: return "LVM_DELETECOLUMN";
        case 0x1000 + 29: return "LVM_GETCOLUMNWIDTH";
        case 0x1000 + 30: return "LVM_SETCOLUMNWIDTH";
        case 0x1000 + 31: return "LVM_GETHEADER";
        case 0x1000 + 33: return "LVM_CREATEDRAGIMAGE";
        case 0x1000 + 34: return "LVM_GETVIEWRECT";
        case 0x1000 + 35: return "LVM_GETTEXTCOLOR";
        case 0x1000 + 36: return "LVM_SETTEXTCOLOR";
        case 0x1000 + 37: return "LVM_GETTEXTBKCOLOR";
        case 0x1000 + 38: return "LVM_SETTEXTBKCOLOR";
        case 0x1000 + 39: return "LVM_GETTOPINDEX";
        case 0x1000 + 40: return "LVM_GETCOUNTPERPAGE";
        case 0x1000 + 41: return "LVM_GETORIGIN";
        case 0x1000 + 42: return "LVM_UPDATE";
        case 0x1000 + 43: return "LVM_SETITEMSTATE";
        case 0x1000 + 44: return "LVM_GETITEMSTATE";
        case 0x1000 + 45: return "LVM_GETITEMTEXTA";
        case 0x1000 + 115: return "LVM_GETITEMTEXTW";
        case 0x1000 + 46: return "LVM_SETITEMTEXTA";
        case 0x1000 + 116: return "LVM_SETITEMTEXTW";
        case 0x1000 + 47: return "LVM_SETITEMCOUNT";
        case 0x1000 + 48: return "LVM_SORTITEMS";
        case 0x1000 + 49: return "LVM_SETITEMPOSITION32";
        case 0x1000 + 50: return "LVM_GETSELECTEDCOUNT";
        case 0x1000 + 51: return "LVM_GETITEMSPACING";
        case 0x1000 + 52: return "LVM_GETISEARCHSTRINGA";
        case 0x1000 + 117: return "LVM_GETISEARCHSTRINGW";
        case 0x1000 + 53: return "LVM_SETICONSPACING";
        case 0x1000 + 54: return "LVM_SETEXTENDEDLISTVIEWSTYLE";
        case 0x1000 + 55: return "LVM_GETEXTENDEDLISTVIEWSTYLE";
        case 0x1000 + 56: return "LVM_GETSUBITEMRECT";
        case 0x1000 + 57: return "LVM_SUBITEMHITTEST";
        case 0x1000 + 58: return "LVM_SETCOLUMNORDERARRAY";
        case 0x1000 + 59: return "LVM_GETCOLUMNORDERARRAY";
        case 0x1000 + 60: return "LVM_SETHOTITEM";
        case 0x1000 + 61: return "LVM_GETHOTITEM";
        case 0x1000 + 62: return "LVM_SETHOTCURSOR";
        case 0x1000 + 63: return "LVM_GETHOTCURSOR";
        case 0x1000 + 64: return "LVM_APPROXIMATEVIEWRECT";
        case 0x1000 + 65: return "LVM_SETWORKAREA";

        // tree view
        case 0x1100 + 0: return "TVM_INSERTITEMA";
        case 0x1100 + 50: return "TVM_INSERTITEMW";
        case 0x1100 + 1: return "TVM_DELETEITEM";
        case 0x1100 + 2: return "TVM_EXPAND";
        case 0x1100 + 4: return "TVM_GETITEMRECT";
        case 0x1100 + 5: return "TVM_GETCOUNT";
        case 0x1100 + 6: return "TVM_GETINDENT";
        case 0x1100 + 7: return "TVM_SETINDENT";
        case 0x1100 + 8: return "TVM_GETIMAGELIST";
        case 0x1100 + 9: return "TVM_SETIMAGELIST";
        case 0x1100 + 10: return "TVM_GETNEXTITEM";
        case 0x1100 + 11: return "TVM_SELECTITEM";
        case 0x1100 + 12: return "TVM_GETITEMA";
        case 0x1100 + 62: return "TVM_GETITEMW";
        case 0x1100 + 13: return "TVM_SETITEMA";
        case 0x1100 + 63: return "TVM_SETITEMW";
        case 0x1100 + 14: return "TVM_EDITLABELA";
        case 0x1100 + 65: return "TVM_EDITLABELW";
        case 0x1100 + 15: return "TVM_GETEDITCONTROL";
        case 0x1100 + 16: return "TVM_GETVISIBLECOUNT";
        case 0x1100 + 17: return "TVM_HITTEST";
        case 0x1100 + 18: return "TVM_CREATEDRAGIMAGE";
        case 0x1100 + 19: return "TVM_SORTCHILDREN";
        case 0x1100 + 20: return "TVM_ENSUREVISIBLE";
        case 0x1100 + 21: return "TVM_SORTCHILDRENCB";
        case 0x1100 + 22: return "TVM_ENDEDITLABELNOW";
        case 0x1100 + 23: return "TVM_GETISEARCHSTRINGA";
        case 0x1100 + 64: return "TVM_GETISEARCHSTRINGW";
        case 0x1100 + 24: return "TVM_SETTOOLTIPS";
        case 0x1100 + 25: return "TVM_GETTOOLTIPS";

        // header
        case 0x1200 + 0: return "HDM_GETITEMCOUNT";
        case 0x1200 + 1: return "HDM_INSERTITEMA";
        case 0x1200 + 10: return "HDM_INSERTITEMW";
        case 0x1200 + 2: return "HDM_DELETEITEM";
        case 0x1200 + 3: return "HDM_GETITEMA";
        case 0x1200 + 11: return "HDM_GETITEMW";
        case 0x1200 + 4: return "HDM_SETITEMA";
        case 0x1200 + 12: return "HDM_SETITEMW";
        case 0x1200 + 5: return "HDM_LAYOUT";
        case 0x1200 + 6: return "HDM_HITTEST";
        case 0x1200 + 7: return "HDM_GETITEMRECT";
        case 0x1200 + 8: return "HDM_SETIMAGELIST";
        case 0x1200 + 9: return "HDM_GETIMAGELIST";
        case 0x1200 + 15: return "HDM_ORDERTOINDEX";
        case 0x1200 + 16: return "HDM_CREATEDRAGIMAGE";
        case 0x1200 + 17: return "HDM_GETORDERARRAY";
        case 0x1200 + 18: return "HDM_SETORDERARRAY";
        case 0x1200 + 19: return "HDM_SETHOTDIVIDER";

        // tab control
        case 0x1300 + 2: return "TCM_GETIMAGELIST";
        case 0x1300 + 3: return "TCM_SETIMAGELIST";
        case 0x1300 + 4: return "TCM_GETITEMCOUNT";
        case 0x1300 + 5: return "TCM_GETITEMA";
        case 0x1300 + 60: return "TCM_GETITEMW";
        case 0x1300 + 6: return "TCM_SETITEMA";
        case 0x1300 + 61: return "TCM_SETITEMW";
        case 0x1300 + 7: return "TCM_INSERTITEMA";
        case 0x1300 + 62: return "TCM_INSERTITEMW";
        case 0x1300 + 8: return "TCM_DELETEITEM";
        case 0x1300 + 9: return "TCM_DELETEALLITEMS";
        case 0x1300 + 10: return "TCM_GETITEMRECT";
        case 0x1300 + 11: return "TCM_GETCURSEL";
        case 0x1300 + 12: return "TCM_SETCURSEL";
        case 0x1300 + 13: return "TCM_HITTEST";
        case 0x1300 + 14: return "TCM_SETITEMEXTRA";
        case 0x1300 + 40: return "TCM_ADJUSTRECT";
        case 0x1300 + 41: return "TCM_SETITEMSIZE";
        case 0x1300 + 42: return "TCM_REMOVEIMAGE";
        case 0x1300 + 43: return "TCM_SETPADDING";
        case 0x1300 + 44: return "TCM_GETROWCOUNT";
        case 0x1300 + 45: return "TCM_GETTOOLTIPS";
        case 0x1300 + 46: return "TCM_SETTOOLTIPS";
        case 0x1300 + 47: return "TCM_GETCURFOCUS";
        case 0x1300 + 48: return "TCM_SETCURFOCUS";
        case 0x1300 + 49: return "TCM_SETMINTABWIDTH";
        case 0x1300 + 50: return "TCM_DESELECTALL";

        // toolbar
        case WM_USER+1: return "TB_ENABLEBUTTON";
        case WM_USER+2: return "TB_CHECKBUTTON";
        case WM_USER+3: return "TB_PRESSBUTTON";
        case WM_USER+4: return "TB_HIDEBUTTON";
        case WM_USER+5: return "TB_INDETERMINATE";
        case WM_USER+9: return "TB_ISBUTTONENABLED";
        case WM_USER+10: return "TB_ISBUTTONCHECKED";
        case WM_USER+11: return "TB_ISBUTTONPRESSED";
        case WM_USER+12: return "TB_ISBUTTONHIDDEN";
        case WM_USER+13: return "TB_ISBUTTONINDETERMINATE";
        case WM_USER+17: return "TB_SETSTATE";
        case WM_USER+18: return "TB_GETSTATE";
        case WM_USER+19: return "TB_ADDBITMAP";
        case WM_USER+20: return "TB_ADDBUTTONS";
        case WM_USER+21: return "TB_INSERTBUTTON";
        case WM_USER+22: return "TB_DELETEBUTTON";
        case WM_USER+23: return "TB_GETBUTTON";
        case WM_USER+24: return "TB_BUTTONCOUNT";
        case WM_USER+25: return "TB_COMMANDTOINDEX";
        case WM_USER+26: return "TB_SAVERESTOREA";
        case WM_USER+76: return "TB_SAVERESTOREW";
        case WM_USER+27: return "TB_CUSTOMIZE";
        case WM_USER+28: return "TB_ADDSTRINGA";
        case WM_USER+77: return "TB_ADDSTRINGW";
        case WM_USER+29: return "TB_GETITEMRECT";
        case WM_USER+30: return "TB_BUTTONSTRUCTSIZE";
        case WM_USER+31: return "TB_SETBUTTONSIZE";
        case WM_USER+32: return "TB_SETBITMAPSIZE";
        case WM_USER+33: return "TB_AUTOSIZE";
        case WM_USER+35: return "TB_GETTOOLTIPS";
        case WM_USER+36: return "TB_SETTOOLTIPS";
        case WM_USER+37: return "TB_SETPARENT";
        case WM_USER+39: return "TB_SETROWS";
        case WM_USER+40: return "TB_GETROWS";
        case WM_USER+42: return "TB_SETCMDID";
        case WM_USER+43: return "TB_CHANGEBITMAP";
        case WM_USER+44: return "TB_GETBITMAP";
        case WM_USER+45: return "TB_GETBUTTONTEXTA";
        case WM_USER+75: return "TB_GETBUTTONTEXTW";
        case WM_USER+46: return "TB_REPLACEBITMAP";
        case WM_USER+47: return "TB_SETINDENT";
        case WM_USER+48: return "TB_SETIMAGELIST";
        case WM_USER+49: return "TB_GETIMAGELIST";
        case WM_USER+50: return "TB_LOADIMAGES";
        case WM_USER+51: return "TB_GETRECT";
        case WM_USER+52: return "TB_SETHOTIMAGELIST";
        case WM_USER+53: return "TB_GETHOTIMAGELIST";
        case WM_USER+54: return "TB_SETDISABLEDIMAGELIST";
        case WM_USER+55: return "TB_GETDISABLEDIMAGELIST";
        case WM_USER+56: return "TB_SETSTYLE";
        case WM_USER+57: return "TB_GETSTYLE";
        case WM_USER+58: return "TB_GETBUTTONSIZE";
        case WM_USER+59: return "TB_SETBUTTONWIDTH";
        case WM_USER+60: return "TB_SETMAXTEXTROWS";
        case WM_USER+61: return "TB_GETTEXTROWS";
        case WM_USER+41: return "TB_GETBITMAPFLAGS";

#endif //WIN32

        default:
            sprintf(s_szBuf, "<unknown message = %d>", message);
            return s_szBuf;
    }
}

HB_FUNC ( W_DECODEMESSAGENAME )
{
    int nMessage = (int) hb_parni(1);
    hb_retc( DecodeMessageName( nMessage ) );
}