/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
* PRG application CLASS & start functions
*/

#include "woopgui.ch"
#include "windows.ch"
#include "common.ch"
#include "hbclass.ch"

#define APPNAME "Applicazione di prova"

STATIC oMyApplication

// Environment application definitions
CLASS TApplication 
    // Base
    PROTECTED:
    DATA cId         AS CHAR
    DATA cName       AS CHAR                 // Application name
    DATA nHandle     INIT GetModuleHandle()  // Application Handle. Must be used from all application abjects.

    DATA cIcon       AS CHAR                 // Application Icon
    DATA cCursor     AS CHAR                 // Application Handle to the class cursor. This member must be a handle to a cursor resource.
    DATA nBkColor    AS NUMERIC              // Application default Background color
    DATA cIconSm     AS CHAR                 // Application small icon
    
    DATA oFrmClass   AS OBJECT
    DATA oDlgClass   AS OBJECT

    DATA bInit       AS CODEBLOCK            // Code block to evaluate on Init
    DATA bExit       AS CODEBLOCK            // Code block to evaluate before Exit

    DATA aoWinClass   INIT {}  HIDDEN        // Array of application windows class definition
    DATA aoWindows    INIT {}  HIDDEN        // Array of application windows
    DATA oCurrentWindow        HIDDEN        // Current Window
    // Variabile per tavole
    // CLASSDATA Tables
    // Variabile per maint
    // CLASSDATA Maints

    // METODI
    METHOD  New() CONSTRUCTOR
    METHOD  Activate()             EXTERN MainLoop()
    METHOD  Create()
    MESSAGE Exit METHOD OnExit()
    METHOD  FindWindowbyHandle()
    METHOD  FindWinClassByName()
    MESSAGE Init METHOD OnInit()
    METHOD  Quit()
    
    METHOD  SetCurrentWindow()
    METHOD  GetCurrentWindow()
    METHOD  UnsetCurrentWindow()

    // AddWindow viene usato direttamente dalla classe Window
    HIDDEN:
    METHOD AddWindow()   BLOCK {|Self, oWnd| aAdd( ::aoWindows, oWnd ) }
    METHOD DelWindow()   BLOCK {|Self, oWnd, nPos| iif( nPos := AScan( ::aoWindows, oWnd ), aDel( ::aoWindows, nPos ), NIL ) }
    METHOD AddWinClass() BLOCK {|Self, oWnd| aAdd( ::aoWinClass, oWnd ) }
    METHOD UnregisterWinClasses() 

    //ON ERROR ErrorHandler()

ENDCLASS

FUNCTION ApplObj()
RETURN oMyApplication

FUNCTION ApplHandle()
RETURN oMyApplication:nHandle

METHOD New( cId, cName, cIcon, cCursor, nBkColor, cIconSm, bInit, bExit, bFrmEvents, bDlgEvents )
    LOCAL oWinDef
    LOCAL n, pIcon
    
    IF oMyApplication == NIL

       IF cId == NIL
          MessageBox( , "Errore. Manca l'Identificativo dell'Applicazione" )
          QUIT
       ENDIF

       ASSIGN ::cId        WITH cId
       ASSIGN ::cName      WITH cName     DEFAULT "Applicazione di prova"
       ASSIGN ::cIcon      WITH cIcon
       ASSIGN ::cCursor    WITH cCursor
       ASSIGN ::nBkColor   WITH nBkColor
       ASSIGN ::cIconSm    WITH cIconSm    DEFAULT ::cIcon
       ASSIGN ::bInit      WITH bInit
       ASSIGN ::bExit      WITH bExit

       // Definisco l'ambiente e attivo la finestra principale
       ::aoWinClass := {}
       ::aoWindows  := {}
       
       oMyApplication := Self

       // Inizializzo i controlli
       InitCommonControlsEx()
    ELSE
       Self := oMyApplication
    ENDIF

RETURN Self

METHOD Create()
   LOCAL oWinDef

   IF ::oFrmClass == NIL
      // Define Standard Application Form Window
      oWinDef := TWindowDef():New( ,, ::nHandle )
      oWinDef:cClassName := "WoopGUIFrmClass"
      
      IF ::cIcon   <> NIL THEN oWinDef:SetIconFromFile( ::cIcon )
      IF ::cCursor <> NIL THEN oWinDef:SetCursorFromFile( ::cCursor )
      IF ::cIconSm <> NIL 
         IF ::cIconSm == ::cIcon
            oWinDef:hIconSm := oWinDef:hIcon
         ELSE
            oWinDef:SetIconSmFromFile( ::cIcon )
         ENDIF
      ENDIF
                               
      oWinDef:Register()
      ::oFrmClass := oWinDef
      ::AddWinClass( oWinDef )
   ENDIF

   IF ::oDlgClass == NIL
      // Define Standard Application Dialog Window
      oWinDef := TWindowDef():New( ,, ::nHandle )
      oWinDef:cClassName := "WoopGUIDlgClass"
      oWinDef:bWndProc   := {|hWnd, message, lParam, wParam| W_DlgEvents(hWnd, message, lParam, wParam) }
      
      // IF ::cIcon   <> NIL THEN oWinDef:SetIconFromFile( ::cIcon )
      // IF ::cCursor <> NIL THEN oWinDef:SetCursorFromFile( ::cCursor )
      // IF ::cIconSm <> NIL 
      //    IF ::cIconSm == ::cIcon
      //       oWinDef:hIconSm := oWinDef:hIcon
      //    ELSE
      //       oWinDef:SetIconSmFromFile( ::cIcon )
      //    ENDIF
      // ENDIF

      oWinDef:Register()
      ::oDlgClass := oWinDef
      ::AddWinClass( oWinDef )
   ENDIF
   
RETURN Self

METHOD FindWindowbyHandle( nHandle )
   LOCAL oWin
   LOCAL nPos := aScan( ::aoWindows, {|e| e:nHandle == nHandle } )
   IF nPos > 0 THEN oWin := ::aoWindows[ nPos ]
RETURN oWin

METHOD FindWinClassByName( cClassName )
   LOCAL oWin
   LOCAL nPos
   cClassName := Upper( cClassName )
   nPos := aScan( ::aoWinClass, {|e| Upper( e:cClassName ) == cClassName } )
   IF nPos > 0 THEN oWin := ::aoWinClass[ nPos ]
RETURN oWin

METHOD GetCurrentWindow()
RETURN ::oCurrentWindow

METHOD OnInit()
   // Procedure prima dell'avvio dell'applicazione
RETURN Self

METHOD OnExit()
   // Procedure al termine dell'applicazione
RETURN Self

METHOD SetCurrentWindow( oWnd )
  LOCAL nOldWnd := ::oCurrentWindow
  ::oCurrentWindow := oWnd
RETURN nOldWnd

METHOD Quit()
  IF !Empty( ::aoWindows )
     AEval( ::aoWindows, {|o| o:Destroy() } )
  ENDIF
RETURN ::UnRegisterWinClasses()

METHOD UnregisterWinClasses()
  LOCAL lOk := TRUE, i, oWin
  FOR i := 1 TO Len( ::aoWinClass )
     oWin := ::aoWinClass[i]
     lOk := UnRegisterClass( oWin:cClassName, ::nHandle )
     IF !lOk
        EXIT
     ENDIF
  NEXT i
  //aEval( ::aoWinClass, {|oWin| lOk := UnRegisterClass( oWin:cClassName, ::nHandle ) } )
RETURN lOk

METHOD UnsetCurrentWindow( oWnd )
  ::oCurrentWindow := NIL
RETURN NIL

/*
METHOD ErrorHandler( x )
   // Gestione errori della classe

   if PCount() > 0
      W_MsgBox( x, "Errore" )
   endif

   MsgBox( __GetMessage(), "Errore" )  // Shows the message that was sent to the object

RETURN
*/
// ----------------------------------------------------
//
// Funzioni di servizio
//
// ----------------------------------------------------

#include "error.ch"
FUNCTION GenMyError( cErrore )
   LOCAL oErr                                        
   // Qui la gestione errori                         
   //MessageBox( , "Genero file di log (parametri"+str(PCOUNT())+")" )
   oErr := ErrorNew()                                
      oErr:severity    := ES_ERROR                   
      oErr:genCode     := EG_UNSUPPORTED
      oErr:subSystem   := "BASE"                     
      oErr:SubCode     := 0
      oErr:Description := cErrore          
      Eval( ErrorBlock(), oErr )                     
                                                     
RETURN TRUE

FUNCTION ObjDisplayData( oB )
   local cString := "", i, aData

      cString += "Object Name: " + oB:ClassName + CRLF
      
      aData := __objGetValueList( oB )
      FOR i = 1 to len ( aData )
          cString += "DATA name: " + Pad( aData[ i, HB_OO_DATA_SYMBOL ], 25 )
          cString += " - value: " + Pad( Char( aData[ i, HB_OO_DATA_VALUE  ] ), 30 )
          cString += CRLF
      NEXT
   
   MessageBox(, cString, "Object Data" )

RETURN 0

FUNCTION ParamDisplay( oB, aParam )
   local cString := "", i, aData

      cString += "Object Name: " + oB:ClassName + CRLF
      
      aData := aParam
      FOR i = 1 to len ( aData )
          cString += "Param: " + str(i,2)
          cString += " - Type: " + ValType( aData[i] )
          cString += " - value= " + Char( aData[ i ] )
          cString += CRLF
      NEXT
   
   MessageBox(, cString, "Parameters Data" )

RETURN 0

