/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
* From this class descends application window
*/

#include "woopgui.ch"
#include "common.ch"
#include "hbclass.ch"
#include "windows.ch"

CLASS TDialog FROM TWindow

    // Base
    DATA aoChilds AS ARRAY INIT {} HIDDEN

    // Data of this class only
    //DATA oMenu        HIDDEN
    DATA oStatusBar AS OBJECT HIDDEN

    // Public Methods
    METHOD New() CONSTRUCTOR

    METHOD CreateStatusBar()
    METHOD GetStatusBar()
    METHOD SetStatusBar()

    // Hidden methods
    HIDDEN:
    METHOD AddChild()        BLOCK {|Self, oCtr| aAdd( ::aoChilds, oCtr ) }
    //METHOD DelControl() BLOCK {|Self, oCtr, nPos| iif( nPos := AScan( ::aoControls, oCtr ), aDel( ::aoControls, nPos ), NIL ) }
    METHOD GetChildren()     INLINE ::aoChilds
    METHOD Destroy()
    METHOD DestroyChildren() BLOCK {|Self| aEval( ::aoChilds, {|x| x:Destroy() } ) }
    //METHOD FindMenuItem()


ENDCLASS

METHOD New( cName, nRow, nCol, nWidth, nHeight, oParent, lStatusBar )

    // window defaults
    DEFAULT lStatusBar TO TRUE
    
    ASSIGN ::nExStyle     WITH WS_EX_DLGMODALFRAME
    ASSIGN ::cClassName   WITH "WoopGUIDlgClass"
    ASSIGN ::cName        WITH cName                DEFAULT "Dialog_1"
    ASSIGN ::nStyle       WITH WS_TABSTOP + WS_DLGFRAME
    ASSIGN ::nRow         WITH nRow                 DEFAULT CW_USEDEFAULT
    ASSIGN ::nCol         WITH nCol                 DEFAULT CW_USEDEFAULT
    ASSIGN ::nWidth       WITH nWidth               DEFAULT CW_USEDEFAULT
    ASSIGN ::nHeight      WITH nHeight              DEFAULT CW_USEDEFAULT
    ASSIGN ::nChild       WITH NULL
    ASSIGN ::nApplication WITH NULL // ApplObj()
    ASSIGN ::pStruct      WITH NULL

    ::Super:New( ::nExStyle, ::cClassName, ::cName, ::nStyle, ;
                 ::nRow, ::nCol, ::nWidth, ::nHeight, ;
                 oParent, ::nChild, ::nApplication, ::pStruct )
    
    ::oWindowDef := ApplObj():oDlgClass
    ::SetEventHandler( ::oWindowDef:bWndProc )
    
    IF lStatusBar
       // Create SIMPLE status bar
       // FSG - Define application variable to make a standard statusbar
       ::CreateStatusBar()
    ENDIF

RETURN Self

METHOD CreateStatusBar( nParts, nStyle )
  ::oStatusBar := tStatusBar():New( Self, nParts, nStyle )
RETURN IIF( ::oStatusBar <> NIL, TRUE, FALSE )

METHOD Destroy()
   ::DestroyChildren()
RETURN DestroyWindow( ::nHandle )


METHOD GetStatusBar()
RETURN ::oStatusBar

METHOD SetStatusBar( cString, nPart, uFlags )
RETURN ::oStatusBar:SetValue( cString, nPart, uFlags )

// reserved methods  ------------------------------------------

function W_WriteEventsLog( cText,fname )
Local nHand, lWrite := FALSE

  IF lWrite
     fname := Iif( fname == Nil,"events.log",fname )
     if !File( fname )
        nHand := Fcreate( fname )
     else
        nHand := Fopen( fname,1 )
     endif
     Fseek( nHand,0,2 )
     Fwrite( nHand, cText + chr(13)+chr(10) )
     Fclose( nHand )
  ENDIF
  
return nil

FUNCTION W_DlgEvents( hWnd, nMsg, wParam, lParam )
  Local i,procname,c,z,x,ContrlCount,FormCount

    //PAINTSTRUCT ps;
    //HDC hdc;
    LOCAL wmId, wmEvent, wmHandle
    LOCAL nMouseState
    LOCAL xPos, yPos
    LOCAL oWin   // Oggetto Window o Controllo
    LOCAL oItem, oParent
    LOCAL aoWindows := ApplObj():aoWindows // Array of windows and controls
    LOCAL oWinF := ApplObj():FindWindowByHandle( hWnd )

    W_WriteEventsLog( "Window: "+Str(hWnd)+" Name: "+oWinF:cName+"- Messaggio DLG : "+Pad(W_DecodeMessageName( nMsg ),20)+;
                      " wParam: "+Str(wParam,15)+" lParam"+Str(lParam,15))

    do case

      // case nMsg == WM_MENUCOMMAND (WINVER >= 5.00)
      case nMsg == WM_COMMAND
         wmId    = LOWORD(wParam) // menu identifier
         wmEvent = HIWORD(wParam) // 0 = menu, 1 = accelerator
         wmHandle = lParam        // control handle


         //MessageBox( , "Passato" )

         if wmEvent == 0    // Command from menu or window

            if IsWindow( wmHandle ) // Is from a window

               i := aScan( aoWindows, {|W| W:nHandle == wmHandle } )

               //MessageBox( , "Finestra I = " + STR(I) )

               IF i > 0
                  oWin := aoWindows[i]
                  IF oWin:HasAction()
                     //MessageBox( , "Passato" )
                     oWin:ExecAction( oWin )
                  ENDIF
               ENDIF

            endif

         endif

         return 0;

      case nMsg == WM_ACTIVATE
         wmId    = LOWORD(wParam) // Remember, these are...
         wmEvent = HIWORD(wParam) // ...different for Win32!

         do case
            case wmId == WA_ACTIVE
            case wmId == WA_CLICKACTIVE
            case wmId == WA_INACTIVE
         endcase

         i := aScan( aoWindows, hWnd )

         IF i > 0
            oWin := aoWindows[i]
            IF ValType( oWin:bActivate ) == "B"
               Eval( oWin:bActivate, oWin )
            ENDIF
         ENDIF


      *case nMsg == WM_ACTIVATEAPP
      *case nMsg == WM_CANCELMODE
      *case nMsg == WM_CHILDACTIVATE
      case nMsg == WM_CLOSE

      *case nMsg == WM_COMPACTING
      *case nMsg == WM_CREATE
      case nMsg == WM_DESTROY  
           oWinF:Destroy()
           // IF oWinF:oParent <> NIL
           //    oWinF:oParent:Show()
           // ENDIF

      *case nMsg == WM_ENABLE
      *case nMsg == WM_ENTERSIZEMOVE
      *case nMsg == WM_EXITSIZEMOVE
      *case nMsg == WM_GETFONT
      *case nMsg == WM_GETICON
      *case nMsg == WM_GETMINMAXINFO
      *case nMsg == WM_GETTEXT
      *case nMsg == WM_GETTEXTLENGTH
      *case nMsg == WM_INPUTLANGCHANGE
      *case nMsg == WM_INPUTLANGCHANGEREQUEST
      *case nMsg == WM_MOVE
      *case nMsg == WM_MOVING
      *case nMsg == WM_NCACTIVATE
      *case nMsg == WM_NCCALCSIZE
      *case nMsg == WM_NCCREATE
      *case nMsg == WM_NCDESTROY
      *case nMsg == WM_NULL
      *case nMsg == WM_PAINT
      *case nMsg == WM_PARENTNOTIFY
      *case nMsg == WM_QUERYDRAGICON
      *case nMsg == WM_QUERYOPEN
      case nMsg == WM_QUIT
      *case nMsg == WM_SETFONT
      *case nMsg == WM_SETICON

      case nMsg == WM_SETFOCUS


      *case nMsg == WM_SETTEXT
      *case nMsg == WM_SHOWWINDOW
      case nMsg == WM_SIZE

         // Status bar repaint check
         oItem := ApplObj():FindWindowbyHandle( hwnd )
         IF oItem <> NIL
            // Has object a status bar ?
            IF __objHasData( oItem, "OSTATUSBAR" )
               // If so, is initialized
               IF ValType( oItem:oStatusBar ) == "O"
                  // Ok. Repaint.
                  oItem:oStatusBar:Repaint()
               ENDIF
            ENDIF
         ENDIF

      *case nMsg == WM_SIZING
      *case nMsg == WM_STYLECHANGED
      *case nMsg == WM_STYLECHANGING
      *case nMsg == WM_THEMECHANGED
      *case nMsg == WM_USERCHANGED
      *case nMsg == WM_WINDOWPOSCHANGED
      *case nMsg == WM_WINDOWPOSCHANGING

      case  nMsg == WM_LBUTTONDOWN
         nMouseState = wParam
         xPos = GET_X_LPARAM(lParam)
         yPos = GET_Y_LPARAM(lParam)

         // Find window
         i := aScan( aoWindows, hWnd )
         IF i > 0
            oWin := aoWindows[i]
            IF ValType( oWin:bOnLeftClick ) == "B"
               Eval( oWin:bOnLeftClick )
            ENDIF
         ENDIF

	    case  nMsg == WM_CONTEXTMENU

// FSG - from Roberto Lopez MiniGUI - change for WoopGUI
//
//		I := Ascan ( _aFormhandles , hWnd )
//		if i > 0
//			if _aFormContextMenuHandle [i] != 0
//				TrackPopupMenu ( _aFormContextMenuHandle [i]  , LOWORD(lparam) , HIWORD(lparam) , hWnd )
//			Endif
//		EndIf

	    case  nMsg == WM_TIMER

// FSG - from Roberto Lopez MiniGUI - change for WoopGUI
//
//		i := Ascan ( _aControlIds , wParam )
//
//		if i > 0
//			procname = _aControlProcedures [i]
//			if ValType( procname ) == "C"
//			       do &procname
//			else
//			       Eval( procname )
//			Endif
//		EndIf

     endcase

return (-1)
