/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
*/

#include "woopgui.ch"
#include "common.ch"
#include "hbclass.ch"
#include "windows.ch"
//#include "wingdi.ch"

// Menu definitions
CLASS TFont
    // Base
    DATA nHandle            AS NUMERIC    // Handle
    DATA nHeight            AS NUMERIC    // height of font
    DATA nWidth             AS NUMERIC    // average character width
    DATA nEscapement        AS NUMERIC    // angle of escapement
    DATA nOrientation       AS NUMERIC    // base-line orientation angle
    DATA nWeight            AS NUMERIC    // font weight
    DATA lItalic            AS LOGICAL    // italic attribute option
    DATA lUnderline         AS LOGICAL    // underline attribute option
    DATA lStrikeOut         AS LOGICAL    // strikeout attribute option
    DATA nCharSet           AS NUMERIC    // character set identifier
    DATA nOutputPrecision   AS NUMERIC    // output precision
    DATA nClipPrecision     AS NUMERIC    // clipping precision
    DATA nQuality           AS NUMERIC    // output quality
    DATA nPitchAndFamily    AS NUMERIC    // pitch and family
    DATA cFace              AS STRING     // typeface name

    // METODI
    METHOD  New() CONSTRUCTOR
    METHOD  Create()
    METHOD  GetValue() 
    METHOD  CreateExtended()
    METHOD  SetBold()
    
ENDCLASS

METHOD New( cFontFace AS STRING, nFontSize AS NUMERIC, lItalic, lUnderline, lStrikeOut, lBold )

    DEFAULT lBold TO FALSE
    
    ASSIGN ::nHeight   WITH ConvertPointsToLU( nFontSize ) DEFAULT 0
    ::nWidth           := 0
    ::nEscapement      := 0
    ::nOrientation     := 0
    ::nWeight          := IIF( lBold, FW_NORMAL, FW_BOLD )
    ASSIGN ::lItalic    WITH lItalic    DEFAULT FALSE
    ASSIGN ::lUnderline WITH lUnderline DEFAULT FALSE
    ASSIGN ::lStrikeOut WITH lStrikeOut DEFAULT FALSE
    ::nCharSet         := DEFAULT_CHARSET 
    ::nOutputPrecision := OUT_DEFAULT_PRECIS
    ::nClipPrecision   := CLIP_DEFAULT_PRECIS
    ::nQuality         := DEFAULT_QUALITY
    ::nPitchAndFamily  := DEFAULT_PITCH 
    ASSIGN ::cFace     WITH cFontFace DEFAULT "Arial"
    
    ::Create()
RETURN Self

    
METHOD CreateExtended( nHeight, nWidth, nEscapement, nOrientation, nWeight, lItalic, lUnderline,;
                       lStrikeOut, nCharSet, nOutputPrecision, nClipPrecision, nQuality, nPitchAndFamily,;
                       cFace )

    ASSIGN ::nHeight          WITH nHeight          DEFAULT 0
    ASSIGN ::nWidth           WITH nWidth           DEFAULT 0
    ASSIGN ::nEscapement      WITH nEscapement      DEFAULT 0
    ASSIGN ::nOrientation     WITH nOrientation     DEFAULT 0
    ASSIGN ::nWeight          WITH nWeight          DEFAULT FW_DONTCARE
    ASSIGN ::lItalic          WITH lItalic          DEFAULT FALSE
    ASSIGN ::lUnderline       WITH lUnderline       DEFAULT FALSE
    ASSIGN ::lStrikeOut       WITH lStrikeOut       DEFAULT FALSE
    ASSIGN ::nCharSet         WITH nCharSet         DEFAULT DEFAULT_CHARSET 
    ASSIGN ::nOutputPrecision WITH nOutputPrecision DEFAULT OUT_DEFAULT_PRECIS
    ASSIGN ::nClipPrecision   WITH nClipPrecision   DEFAULT CLIP_DEFAULT_PRECIS
    ASSIGN ::nQuality         WITH nQuality         DEFAULT DEFAULT_QUALITY
    ASSIGN ::nPitchAndFamily  WITH nPitchAndFamily  DEFAULT DEFAULT_PITCH 
    ASSIGN ::cFace            WITH cFace            DEFAULT "Arial"
    
    ::Create()
RETURN Self

METHOD Create()
    RegisterFont( ::nHeight, ::nWidth, ::nEscapement, ::nOrientation, ::nWeight, ;
                               ::lItalic, ::lUnderline, ::lStrikeOut, ::nCharSet, ::nOutputPrecision,;
                               ::nClipPrecision, ::nQuality, ::nPitchAndFamily, ::cFace )
                               
    ::nHandle := GetRealFontValue( Self )
RETURN ::nHandle

METHOD GetValue()
  ::Create()
RETURN Self

METHOD SetBold( lBold )
  DEFAULT lBold TO TRUE
  ::nWeight          := IIF( lBold, FW_NORMAL, FW_BOLD )
  ::Create()
RETURN Self