/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
* From this class descends application window
*/

#include "woopgui.ch"
#include "common.ch"
#include "hbclass.ch"
#include "windows.ch"

CLASS TFrame FROM TWindow

    // Data of this class only
    DATA oMenu        HIDDEN
    DATA oStatusBar   AS OBJECT HIDDEN

    // Public Methods
    METHOD New()        CONSTRUCTOR
    //METHOD oStatusBar() SETGET
    
    //METHOD GetStatusBar()
    METHOD CreateStatusBar()
    METHOD GetStatusBar()
    METHOD SetStatusBar()

    // Hidden methods
    HIDDEN:
    METHOD AddControl() BLOCK {|Self, oCtr| aAdd( ::aoControls, oCtr ) }
    METHOD DelControl() BLOCK {|Self, oCtr, nPos| iif( nPos := AScan( ::aoControls, oCtr ), aDel( ::aoControls, nPos ), NIL ) }
    METHOD DestroyControls() BLOCK {|Self| aEval( ::aoControls, {|x| x:Destroy() } ) }
    METHOD FindMenuItem()

ENDCLASS

METHOD New( cName, nStyle, nRow, nCol, nWidth, nHeight, oParent, lStatusBar )

    // window defaults
    DEFAULT lStatusBar TO TRUE

    ASSIGN ::cClassName   WITH "WoopGUIFrmClass"
    ASSIGN ::cName        WITH cName                   DEFAULT "Frame_1"          
    ASSIGN ::nStyle       WITH nStyle                  DEFAULT WS_OVERLAPPEDWINDOW 
    ASSIGN ::nRow         WITH nRow                    DEFAULT CW_USEDEFAULT       
    ASSIGN ::nCol         WITH nCol                    DEFAULT CW_USEDEFAULT       
    ASSIGN ::nWidth       WITH nWidth                  DEFAULT CW_USEDEFAULT       
    ASSIGN ::nHeight      WITH nHeight                 DEFAULT CW_USEDEFAULT       
    ASSIGN ::nChild       WITH NULL
    ASSIGN ::nApplication WITH NULL // ApplObj()
    ASSIGN ::pStruct      WITH NULL
    
    ::Super:New( ::nExStyle, ::cClassName, ::cName, ::nStyle, ;
                 ::nRow, ::nCol, ::nWidth, ::nHeight, ;
                 oParent, ::nChild, ::nApplication, ::pStruct )
                 
    IF lStatusBar
       // Create SIMPLE status bar 
       // FSG - Define application variable to make a standard statusbar
       ::CreateStatusBar()
    ENDIF

RETURN Self

//METHOD oStatusBar( oStatusBar AS OBJECT )
//   LOCAL oOldSB := ::myoStatusBar
//   IF oStatusBar <> NIL
//      ::myoStatusBar := oStatusBar
//   ENDIF
//RETURN oOldSB

METHOD CreateStatusBar( nParts, nStyle )
  ::oStatusBar := tStatusBar():New( Self, nParts, nStyle )
RETURN IIF( ::oStatusBar <> NIL, TRUE, FALSE )

METHOD GetStatusBar()
RETURN ::oStatusBar

METHOD SetStatusBar( cString, nPart, uFlags )
RETURN ::oStatusBar:SetValue( cString, nPart, uFlags )

// reserved methods  ------------------------------------------

METHOD FindMenuItem( nID )
   LOCAL oItem

   IF ValType( ::oMenu ) == "O"
      oItem := W_FindMenuItem( ::oMenu, nID )
   ENDIF

RETURN oItem

STATIC FUNCTION W_FindMenuItem( oMenu, nID )
   LOCAL oItem, aoItems
   LOCAL n

   aoItems := oMenu:aoItems

   FOR n := 1 TO Len( aoItems )
       IF ValType( aoItems[n]:nIDItem ) == "O" // There is a submenu
          // Make a recursive call
          IF ( oItem := W_FindMenuItem( aoItems[n]:nIDItem, nID ) ) <> NIL
             EXIT
          ENDIF
       ELSE
          IF aoItems[n]:nIDItem == nID
             oItem := aoItems[n]
             EXIT
          ENDIF
       ENDIF

   NEXT n

RETURN oItem

