/*
*-----------------------------------------------------------------------------
* WoopGUI for Harbour - Win32 OOP GUI library source code
* Copyright 2002 Francesco Saverio Giudice <info@fsgiudice.com>
*
*-----------------------------------------------------------------------------
* Parts of this project come from:
* "Harbour MiniGUI"
*                   Copyright 2002 Roberto Lopez <roblez@ciudad.com.ar>
*                   http://www.geocities.com/harbour_minigui/
* "Harbour GUI framework for Win32"
*                   Copyright 2001 Alexander S.Kresin <alex@belacy.belgorod.su>
*                   Copyright 2001 Antonio Linares <alinares@fivetech.com>
*                   http://www.harbour-project.org
*-----------------------------------------------------------------------------
*
* Base Class - All windows and controls derive from this class
* NOT USE THIS CLASS DIRECTLY !!!
*/

#include "woopgui.ch"
#include "common.ch"
#include "hbclass.ch"
#include "windows.ch"

CLASS TWindowDef

    DATA nStyle         AS NUMERIC      // Specifies the class style(s)
    DATA bWndProc       AS CODEBLOCK    // Pointer to the window procedure
    DATA hInstance      AS NUMERIC      // Handle to the instance that contains the window procedure for the class
    DATA hIcon          AS NUMERIC      // Handle to the class icon. This member must be a handle to an icon resource. If this member is NULL, the system provides a default icon
    DATA hCursor        AS NUMERIC      // Handle to the class cursor. This member must be a handle to a cursor resource. If this member is NULL, an application must explicitly set the cursor shape whenever the mouse moves into the application's window
    DATA hbrBackground  AS NUMERIC      // Handle to the class background brush. This member can be a handle to the physical brush to be used for painting the background, or it can be a color value
    DATA cMenuName      AS STRING       // character string that specifies the resource name of the class menu. If this member is NULL, windows belonging to this class have no default menu
    DATA cClassName     AS STRING       // Specifies the window class name. The class name can be any name registered, or any of the predefined control-class names.
    DATA hIconSm        AS NUMERIC      // Handle to a small icon that is associated with the window class. If this member is NULL, the system searches the icon resource specified by the hIcon member for an icon of the appropriate size to use as the small icon

    // Public methods
    METHOD  New() CONSTRUCTOR
    METHOD  Register()                  // Register Class
    METHOD  SetCursorFromFile()
    METHOD  SetIconFromFile()
    METHOD  SetIconSmFromFile()
    //METHOD  WindowProc()

ENDCLASS

METHOD New( nStyle, bWndProc, hInstance, hIcon, hCursor, hbrBackground, cMenuName, cClassName, hIconSm )
    LOCAL xVal

    // window defaults
    ASSIGN ::nStyle        WITH nStyle         DEFAULT CS_HREDRAW + CS_VREDRAW + CS_OWNDC
    ASSIGN ::bWndProc      WITH bWndProc       DEFAULT {|hWnd, message, lParam, wParam| W_WndEvents(hWnd, message, lParam, wParam) }
    ASSIGN ::hInstance     WITH hInstance      DEFAULT GetModuleHandle()
    ASSIGN ::hIcon         WITH hIcon          DEFAULT LoadIcon(NULL, IDI_APPLICATION)
    ASSIGN ::hCursor       WITH hCursor        DEFAULT LoadCursor(NULL, IDC_ARROW)
    ASSIGN ::hbrBackground WITH hbrBackground  DEFAULT COLOR_BTNFACE
    ASSIGN ::cMenuName     WITH cMenuName
    ASSIGN ::cClassName    WITH cClassName     DEFAULT "WoopGUIClass"
    ASSIGN ::hIconSm       WITH hIconSm        DEFAULT LoadIcon(NULL, IDI_APPLICATION)

    //ObjDisplayData( Self )

RETURN Self

METHOD SetCursorFromFile( cFileName )
   ::hCursor := GetCursorFromFile( ::hInstance, cFileName )
RETURN ::hCursor

METHOD SetIconFromFile( cFileName )
   ::hIcon := GetIconFromFile( ::hInstance, cFileName )
RETURN ::hIcon

METHOD SetIconSmFromFile( cFileName )
   ::hIconSm := GetIconFromFile( ::hInstance, cFileName )
RETURN ::hIconSm

METHOD Register()
  //ObjDisplayData( Self )
RETURN RegisterClassEx( ::nStyle, 0/*bWndProc*/, ::hInstance, ::hIcon, ::hCursor, ::hbrBackground,;
                        ::cMenuName, ::cClassName, ::hIconSm )

STATIC FUNCTION GetCursorFromFile( hInstance, cFileName )
RETURN LoadImage( hInstance, cFileName, IMAGE_CURSOR, 0, 0, LR_LOADFROMFILE + LR_DEFAULTSIZE )

STATIC FUNCTION GetIconFromFile( hInstance, cFileName )
   LOCAL nHandle := LoadImage( hInstance, cFileName, IMAGE_ICON, 0, 0, LR_LOADFROMFILE + LR_DEFAULTSIZE )
   //MessageBox( , "nHandle := " + str( nHandle ) )
RETURN nHandle

//METHOD WindowProc( hWnd, nMessage, wParam, lParam )
//RETURN Eval( ::bWndProc, hWnd, nMessage, wParam, lParam )

// ----------------------------------------------------
//
// W_DefWndEvents( hWnd, nMsg, wParam, lParam ) --> Bool
//
// ----------------------------------------------------
FUNCTION W_DefWndEvents( hWnd, nMsg, wParam, lParam )
  LOCAL oWin, oWinClass
  LOCAL lRet := -1

  // FSG - Must to be optimized !!!!!!!!
  oWin      := ApplObj():FindWindowByHandle( hwnd )
  IF oWin <> NIL
     // oWinClass := ApplObj():FindWinClassByName( oWin:cClassName )
     // IF oWinClass <> NIL
     //    lRet := Eval( oWinClass:bWndProc, hWnd, nMsg, wParam, lParam )
     //    //MessageBox( , "Procedura Finestra eseguita" )
     // ENDIF
     lRet := oWin:WindowProc( nMsg, wParam, lParam )
  ELSE
     // Windows not yet create
  ENDIF
  
  // I return to c call function ( Call DefWindowProc() )
RETURN lRet

/* DA SISTEMARE
 FSG - On Event write to status bar
                 oItem := ApplObj():FindWindowbyHandle( hwnd )
                 IF oItem <> NIL
                    IF __objHasData( oItem, "CSTATUSBAR" )
                       MessageBox( , "Trovata finestra" )
                       IF ValType( oItem:cStatusBar ) == "C"
                          oParent := ApplObj():FindWindowbyHandle( oItem:nParent )
                          IF oParent <> NIL .AND. ValType( oParent:oStatusBar ) == "O"
                             oParent:SetStatusBar( oItem:cStatusBar )
                               //MessageBox( , "Passato" )
                          ENDIF
                       ENDIF
                    ENDIF
                 ENDIF

*/
